package Acme::Test::SysPath;

=head1 NAME

Acme::Test::SysPath - testing distribution for Module::Build::SysPath

=cut

use warnings;
use strict;

our $VERSION = '0.01';

sub hello {
	return 'world';
}

1;
